package com.lzmispt.data.resource.service.metadata.impl;

package .tmp;

import com.lzmispt.data.center.entity.dto.databasestruct.ColumnParamDTO;
import com.lzmispt.data.center.entity.dto.databasestruct.DBColumnDTO;
import com.lzmispt.data.center.entity.dto.databasestruct.DataSourceDTO;
import com.lzmispt.data.center.entity.dto.databasestruct.TableParamDTO;
import com.lzmispt.data.center.entity.dto.log.CommonLogDTO;
import com.lzmispt.data.center.entity.dto.standard.metadata.MetadataEntityDataItemDTO;
import com.lzmispt.data.center.entity.enums.BaseSysFieldEnum;
import com.lzmispt.data.center.entity.enums.StatusCodeEnum;
import com.lzmispt.data.center.entity.vo.resource.datasource.DataSourceVO;
import com.lzmispt.data.center.entity.vo.resource.metadata.InfoItemVO;
import com.lzmispt.data.center.entity.vo.resource.metadata.MetadataFieldVO;
import com.lzmispt.data.center.entity.vo.standard.metadata.MetadataEntityDataItemVO;
import com.lzmispt.data.center.entity.vo.standard.metadata.MetadataFieldWithElementVO;
import com.lzmispt.data.resource.client.DataStandardClient;
import com.lzmispt.data.resource.dao.metadata.IMetadataDao;
import com.lzmispt.data.resource.entity.dto.metadata.AddChildInfoItemDTO;
import com.lzmispt.data.resource.entity.dto.metadata.SaveChildEntityDTO;
import com.lzmispt.data.resource.entity.dto.metadata.SyncChildItemInfosToDbDTO;
import com.lzmispt.data.resource.entity.po.metadata.MetadataEntityInfoItemPO;
import com.lzmispt.data.resource.entity.po.metadata.MetadataEntityPO;
import com.lzmispt.data.resource.service.metadata.IComplexInfoItem;
import com.lzmispt.data.resource.service.metadata.IDataResource;
import com.lzmispt.data.resource.service.metadata.IInfoItem;
import com.lzmispt.data.resource.service.metadata.IInfoItemSync;
import com.lzmispt.data.resource.service.metadata.IMetadataField;
import com.lzmispt.dcbase.ServiceUtil;
import com.lzmispt.dcbase.database.IManage;
import com.lzmispt.dcbase.databasestruct.DataStructProvider;
import com.lzmispt.dcbase.dcconst.DCConst;
import com.lzmispt.dcbase.sysfield.SysFieldHelper;
import com.lzmispt.dcbase.util.HeaderInterceptor;
import com.lzmispt.framework.entity.entity.ResultEntity;
import com.lzmispt.framework.exception.ExceptionUtil;
import com.lzmispt.framework.fconst.FrameConst;
import com.lzmispt.framework.utils.DBKeywordUtil;
import com.lzmispt.framework.utils.RabbitMQUtil;
import com.lzmispt.framework.utils.WorkerId;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import java.util.stream.Collectors;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.cloud.context.config.annotation.RefreshScope;
import org.springframework.util.StringUtils;

@RefreshScope
public abstract class ComplexInfoItemManager implements IComplexInfoItem, IManage {
  private static String DEFAULT_SCHEMA = "dbo";
  
  @Value("${data-center.primary-key-mode}")
  private String primaryKeyMode;
  
  @Autowired
  DataStandardClient dataStandardClient;
  
  @Autowired
  IMetadataField metadataField;
  
  public ResultEntity<Boolean> createChildEntityOrUpdateChildEntity(SaveChildEntityDTO saveChildEntityDTO) {
    String schemaName = "";
    String tableName = saveChildEntityDTO.getTableName();
    MetadataEntityInfoItemPO parentInfoItem = getItemInfoById(saveChildEntityDTO.getParentInfoItemId(), saveChildEntityDTO.getTokenId(), saveChildEntityDTO.getIsRes());
    MetadataEntityPO parentEntity = ((IMetadataDao)ServiceUtil.getMetadataDao(saveChildEntityDTO.getPlatCode(), IMetadataDao.class)).selectEntityByEntityId(saveChildEntityDTO.getParentEntityId());
    if (isSameTableName(parentEntity.getDataSourceId(), tableName, parentInfoItem.getChildEntityId()))
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_DATA_TABLE_EXISTS.getCode()), StatusCodeEnum.DATA_RESOURCE_DATA_TABLE_EXISTS.getMessage()); 
    if (parentEntity != null) {
      schemaName = parentEntity.getDataSchemaName();
      DataSourceVO dataSourceVO = null;
      if (parentEntity.getDataSourceId() != null)
        dataSourceVO = (DataSourceVO)this.dataStandardClient.getDataSourceInfoDoNotDecryptById(parentEntity.getDataSourceId(), saveChildEntityDTO.getPlatCode(), saveChildEntityDTO.getTokenId()).getObj(); 
      if (dataSourceVO == null)
        ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_GET_CONFIG_DB_INSTANCE_NAME_FAILED.getCode()), StatusCodeEnum.DATA_RESOURCE_GET_CONFIG_DB_INSTANCE_NAME_FAILED
            .getMessage()); 
      if (DBKeywordUtil.isDBKeyword(tableName, dataSourceVO.getType()))
        return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_IS_DB_KEYWORDS.getCode()), "【" + tableName + "】是数据库关键字"); 
      DataSourceDTO dataSourceDTO = new DataSourceDTO();
      BeanUtils.copyProperties(dataSourceVO, dataSourceDTO);
      ResultEntity<DataStructProvider> getDataBaseProviderRet = DataStructProvider.getDataBaseProvider(dataSourceDTO);
      if (!getDataBaseProviderRet.getCode().equals(Integer.valueOf(0)))
        return ResultEntity.errorMessage(getDataBaseProviderRet.getCode(), getDataBaseProviderRet.getMsg()); 
      DataStructProvider dataStructProvider = (DataStructProvider)getDataBaseProviderRet.getObj();
      if (dataStructProvider.isExistsTable(tableName).booleanValue())
        return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_DB_EXISTS_TABLE.getCode()), "数据库下已经存在表【" + tableName + "】"); 
    } 
    String[] tableInfo = { schemaName, tableName };
    if (tableName.contains(".")) {
      tableInfo = tableName.split(".");
      tableInfo[0] = StringUtils.isEmpty(tableInfo[0]) ? schemaName : tableInfo[0];
    } 
    if (parentInfoItem != null) {
      MetadataEntityPO childEntity = null;
      if (parentInfoItem.getChildEntityId() != null)
        childEntity = ((IMetadataDao)ServiceUtil.getMetadataDao(saveChildEntityDTO.getPlatCode(), IMetadataDao.class)).selectEntityByEntityId(parentInfoItem.getChildEntityId()); 
      if (childEntity != null) {
        childEntity.setEntityTableName(tableInfo[1]);
        childEntity.setDataSchemaName(tableInfo[0]);
        childEntity.setUpdateTime(new Date());
        ((IMetadataDao)ServiceUtil.getMetadataDao(saveChildEntityDTO.getPlatCode(), IMetadataDao.class)).updateMetadataEntity(childEntity);
      } else if (parentEntity != null) {
        childEntity = new MetadataEntityPO();
        childEntity.setEntityId((Long)WorkerId.getNextId(Long.class));
        childEntity.setEntityName(tableInfo[1]);
        childEntity.setDataSourceId(parentEntity.getDataSourceId());
        childEntity.setDataCategory(parentEntity.getDataCategory());
        childEntity.setDataSchemaName(tableInfo[0]);
        childEntity.setEntityTableName(tableInfo[1]);
        childEntity.setIsMasterEntity(Integer.valueOf(0));
        childEntity.setParentEntityId(saveChildEntityDTO.getParentEntityId());
        childEntity.setIsDelete(Integer.valueOf(0));
        childEntity.setCreateTime(new Date());
        childEntity.setOrganizationId(saveChildEntityDTO.getOrganizationId());
        childEntity.setUserId(saveChildEntityDTO.getUserId());
        ((IMetadataDao)ServiceUtil.getMetadataDao(saveChildEntityDTO.getPlatCode(), IMetadataDao.class)).insertMetadataEntity(childEntity);
        parentInfoItem.setChildEntityId(childEntity.getEntityId());
        ((IMetadataDao)ServiceUtil.getMetadataDao(saveChildEntityDTO.getPlatCode(), IMetadataDao.class)).updateInfoItem(parentInfoItem);
      } 
    } 
    return ResultEntity.successMessage(Boolean.valueOf(true));
  }
  
  public ResultEntity<InfoItemVO> addChildInfoItem(AddChildInfoItemDTO childInfoItemDTO) {
    MetadataEntityInfoItemPO infoItemPO = new MetadataEntityInfoItemPO();
    BeanUtils.copyProperties(childInfoItemDTO, infoItemPO);
    infoItemPO.setInfoItemId((Long)WorkerId.getNextId(Long.class));
    infoItemPO.setIsPrimaryKey(Integer.valueOf(0));
    infoItemPO.setInfoItemStatus(Integer.valueOf(0));
    infoItemPO.setIsDelete(Integer.valueOf(0));
    infoItemPO.setCreateTime(new Date());
    infoItemPO.setInfoItemType(Integer.valueOf(0));
    infoItemPO.setIsReference(DCConst.FALSE);
    ((IMetadataDao)ServiceUtil.getMetadataDao(childInfoItemDTO.getPlatCode(), IMetadataDao.class)).insertEntityInfoItem(infoItemPO);
    InfoItemVO infoItemVO = new InfoItemVO();
    BeanUtils.copyProperties(infoItemPO, infoItemVO);
    return ResultEntity.successMessage(infoItemVO);
  }
  
  public ResultEntity<String> childSyncToDB(SyncChildItemInfosToDbDTO syncChildItemInfosToDbDTO) {
    try {
      Long directoryId = syncChildItemInfosToDbDTO.getDirectoryId();
      List<Long> infoItemIdList = syncChildItemInfosToDbDTO.getItemInfoIds();
      String schemaName = syncChildItemInfosToDbDTO.getSchemaName();
      String tableName = syncChildItemInfosToDbDTO.getTableName();
      return createOrModifyTable(directoryId, infoItemIdList, tableName, schemaName, syncChildItemInfosToDbDTO.getParentInfoItemId(), syncChildItemInfosToDbDTO.getPlatCode(), syncChildItemInfosToDbDTO.getTokenId(), syncChildItemInfosToDbDTO.getIsRes());
    } catch (Exception ex) {
      logErrorInfo(ExceptionUtil.getStackTraceMsg(ex), "子信息项同步异常", syncChildItemInfosToDbDTO.getPlatCode(), syncChildItemInfosToDbDTO.getTokenId());
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_SYNC_CHILD_ITEM_TO_DB_FAILED.getCode()), StatusCodeEnum.DATA_RESOURCE_SYNC_CHILD_ITEM_TO_DB_FAILED.getMessage());
    } 
  }
  
  public ResultEntity<Integer> getChildInfoItemMaxOrder(Long parentInfoItemId, String platCode) {
    MetadataEntityInfoItemPO maxOrderItem = ((IMetadataDao)ServiceUtil.getMetadataDao(platCode, IMetadataDao.class)).selectChildrenInfoItemsMaxOrder(parentInfoItemId);
    if (maxOrderItem != null && maxOrderItem.getInfoItemOrder() != null)
      return ResultEntity.successMessage(Integer.valueOf(maxOrderItem.getInfoItemOrder().intValue() + 1)); 
    return ResultEntity.successMessage(Integer.valueOf(1));
  }
  
  private ResultEntity createOrModifyTable(Long directoryId, List<Long> infoItemIds, String entityTableName, String schemaName, Long pInfoItemId, String platCode, String tokenId, Integer isRes) throws Exception {
    Map<String, Object> mapResult = beforeCreateTable(directoryId, infoItemIds, entityTableName, platCode, pInfoItemId, tokenId, isRes);
    ResultEntity msg = (ResultEntity)mapResult.get("errorMsg");
    if (msg != null && !msg.getCode().equals(Integer.valueOf(0)))
      return msg; 
    MetadataEntityPO entity = (MetadataEntityPO)mapResult.get("entity");
    List<MetadataEntityInfoItemPO> metadataEntityInfoItemCol = (List<MetadataEntityInfoItemPO>)mapResult.get("metadataEntityInfoItemCol");
    String parentItemName = metadataEntityInfoItemCol.stream().filter(item -> Objects.equals(item.getInfoItemId(), pInfoItemId)).map(MetadataEntityInfoItemPO::getInfoItemName).findFirst().orElse("");
    Boolean isSubInfoSync = (Boolean)mapResult.get("isSubInfoSync");
    DataSourceDTO dataSourceDTO = (DataSourceDTO)mapResult.get("dataSourceDTO");
    List<MetadataEntityInfoItemPO> primaryKeyInfoItemCol = (List<MetadataEntityInfoItemPO>)mapResult.get("primaryKeyInfoItemCol");
    DataSourceVO dataSourceVO = null;
    if (entity.getDataSourceId() != null)
      dataSourceVO = (DataSourceVO)this.dataStandardClient.getDataSourceInfoDoNotDecryptById(entity.getDataSourceId(), platCode, tokenId).getObj(); 
    if (dataSourceVO == null)
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_GET_CONFIG_DB_INSTANCE_NAME_BY_SYNC_FAILED.getCode()), StatusCodeEnum.DATA_RESOURCE_GET_CONFIG_DB_INSTANCE_NAME_BY_SYNC_FAILED.getMessage()); 
    dataSourceDTO = new DataSourceDTO();
    BeanUtils.copyProperties(dataSourceVO, dataSourceDTO);
    ResultEntity<DataStructProvider> getDataBaseProviderRet = DataStructProvider.getDataBaseProvider(dataSourceDTO);
    if (!getDataBaseProviderRet.getCode().equals(Integer.valueOf(0)))
      return ResultEntity.errorMessage(getDataBaseProviderRet.getCode(), getDataBaseProviderRet.getMsg()); 
    DataStructProvider dataStructProvider = (DataStructProvider)getDataBaseProviderRet.getObj();
    Boolean isExistMasterTable = dataStructProvider.isExistsTable(entity.getEntityTableName());
    if (!isExistMasterTable.booleanValue())
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_SYNC_ENTITY_UNSYNC_CHILD_UNSYNC_FAILED.getCode()), StatusCodeEnum.DATA_RESOURCE_SYNC_ENTITY_UNSYNC_CHILD_UNSYNC_FAILED.getMessage()); 
    boolean isExistPk = false;
    if (primaryKeyInfoItemCol != null && !primaryKeyInfoItemCol.isEmpty())
      for (int i = 0; i < primaryKeyInfoItemCol.size(); i++) {
        if (!SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.DBID).equalsIgnoreCase(((MetadataEntityInfoItemPO)primaryKeyInfoItemCol.get(i)).getInfoItemName()) && ((MetadataEntityInfoItemPO)primaryKeyInfoItemCol.get(i)).getInfoItemParentId() == null) {
          isExistPk = true;
          break;
        } 
      }  
    if (!isExistPk)
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_SYNC_MAIN_ENTITY_NO_PK_FAILED.getCode()), StatusCodeEnum.DATA_RESOURCE_SYNC_MAIN_ENTITY_NO_PK_FAILED.getMessage()); 
    ResultEntity<List<MetadataEntityInfoItemPO>> getNeedSyncFieldsRet = getNeedSyncFields(entity, metadataEntityInfoItemCol, directoryId, infoItemIds, entityTableName, schemaName, pInfoItemId, platCode, tokenId, isRes);
    if (!getNeedSyncFieldsRet.getCode().equals(Integer.valueOf(0)))
      return getNeedSyncFieldsRet; 
    metadataEntityInfoItemCol = (List<MetadataEntityInfoItemPO>)getNeedSyncFieldsRet.getObj();
    try {
      ResultEntity handleDBFieldRet = null;
      if (!dataStructProvider.isExistsTable(entityTableName).booleanValue()) {
        handleDBFieldRet = handleDBFieldNotExistsTable(entity, metadataEntityInfoItemCol, entityTableName, schemaName, platCode, tokenId, isSubInfoSync, dataStructProvider, primaryKeyInfoItemCol, isRes);
      } else {
        handleDBFieldRet = handleDBFieldExistsTable(entity, metadataEntityInfoItemCol, entityTableName, schemaName, platCode, tokenId, isSubInfoSync, dataStructProvider, primaryKeyInfoItemCol, isRes);
      } 
      setDBFieldDescription(entity, metadataEntityInfoItemCol, schemaName, platCode, tokenId, isSubInfoSync, dataSourceDTO);
      if (!handleDBFieldRet.getCode().equals(Integer.valueOf(0)))
        return handleDBFieldRet; 
      List<String> syncChildName = new ArrayList<>();
      for (MetadataEntityInfoItemPO metadataEntityInfoItemPO : metadataEntityInfoItemCol) {
        Boolean isSysField = ((IInfoItemSync)ServiceUtil.getInfoItemSyncManager(getDefaultCode(), IInfoItemSync.class)).isSystemField(metadataEntityInfoItemPO.getInfoItemName());
        if (!isSysField.booleanValue())
          syncChildName.add(metadataEntityInfoItemPO.getInfoItemName()); 
      } 
      if (!syncChildName.isEmpty())
        logSystemInfo("同步数据资源【" + entity.getEntityName() + "】中复合信息项【" + parentItemName + "】的子信息项【" + StringUtils.join(syncChildName, "、") + "】到数据库成功", platCode, tokenId); 
    } catch (Exception ex) {
      logErrorInfo(ExceptionUtil.getStackTraceMsg(ex), "createOrModifyTable->部分信息项可能同步失败", platCode, tokenId);
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_SYNC_PART_ITEM_FAILED.getCode()), StatusCodeEnum.DATA_RESOURCE_SYNC_PART_ITEM_FAILED.getMessage());
    } 
    return ResultEntity.successMessage(Boolean.valueOf(true));
  }
  
  private Map<String, Object> beforeCreateTable(Long directoryId, List<Long> infoItemIds, String entityTableName, String platCode, Long pInfoItemId, String tokenId, Integer isRes) throws Exception {
    Boolean isSubInfoSync = Boolean.valueOf(false);
    Map<String, Object> result = new HashMap<>(2);
    if (null == infoItemIds || infoItemIds.isEmpty())
      result.put("errorMsg", ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_SYNC_ENTITY_ITEMS_NULL_FAILED.getCode()), StatusCodeEnum.DATA_RESOURCE_SYNC_ENTITY_ITEMS_NULL_FAILED.getMessage())); 
    MetadataEntityPO entity = ((IDataResource)ServiceUtil.getDataResourceManager(getDefaultCode(), IDataResource.class)).thisDirectoryExistMasterEntity(directoryId, platCode);
    DataSourceDTO dataSourceDTO = null;
    if (entity.getDataSourceId() != null) {
      DataSourceVO dataSourceVO = (DataSourceVO)this.dataStandardClient.getDataSourceInfoDoNotDecryptById(entity.getDataSourceId(), platCode, tokenId).getObj();
      if (dataSourceVO == null)
        result.put("errorMsg", ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_GET_CONFIG_DB_INSTANCE_NAME_BY_SYNC_FAILED.getCode()), StatusCodeEnum.DATA_RESOURCE_GET_CONFIG_DB_INSTANCE_NAME_BY_SYNC_FAILED.getMessage())); 
      dataSourceDTO = new DataSourceDTO();
      BeanUtils.copyProperties(dataSourceVO, dataSourceDTO);
    } 
    if (!((IDataResource)ServiceUtil.getDataResourceManager(getDefaultCode(), IDataResource.class)).checkIsValidEntity(entity, tokenId, isRes).booleanValue())
      result.put("errorMsg", ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_SYNC_ENTITY_IS_ERROR_BY_SYNC_FAILED.getCode()), StatusCodeEnum.DATA_RESOURCE_SYNC_ENTITY_IS_ERROR_BY_SYNC_FAILED.getMessage())); 
    List<MetadataEntityInfoItemPO> primaryKeyInfoItemCol = new ArrayList<>();
    List<MetadataEntityInfoItemPO> metadataEntityInfoItemCol = ((IMetadataDao)ServiceUtil.getMetadataDao(platCode, IMetadataDao.class)).getEntityInfoItemsByEntityIdAndQuery(entity.getEntityId(), null);
    if (metadataEntityInfoItemCol != null && !metadataEntityInfoItemCol.isEmpty())
      for (int i = 0; i < metadataEntityInfoItemCol.size(); i++) {
        if (((MetadataEntityInfoItemPO)metadataEntityInfoItemCol.get(i)).getInfoItemParentId() == null && DCConst.TRUE.equals(((MetadataEntityInfoItemPO)metadataEntityInfoItemCol.get(i)).getIsPrimaryKey())) {
          primaryKeyInfoItemCol.add(metadataEntityInfoItemCol.get(i));
        } else if (pInfoItemId.equals(((MetadataEntityInfoItemPO)metadataEntityInfoItemCol.get(i)).getInfoItemParentId()) && DCConst.TRUE.equals(((MetadataEntityInfoItemPO)metadataEntityInfoItemCol.get(i)).getIsPrimaryKey())) {
          primaryKeyInfoItemCol.add(metadataEntityInfoItemCol.get(i));
        } 
      }  
    if (!StringUtils.isEmpty(entityTableName))
      isSubInfoSync = Boolean.valueOf(true); 
    result.put("entity", entity);
    result.put("metadataEntityInfoItemCol", metadataEntityInfoItemCol);
    result.put("isSubInfoSync", isSubInfoSync);
    result.put("dataSourceDTO", dataSourceDTO);
    result.put("primaryKeyInfoItemCol", primaryKeyInfoItemCol);
    return result;
  }
  
  private ResultEntity<List<MetadataEntityInfoItemPO>> getNeedSyncFields(MetadataEntityPO entity, List<MetadataEntityInfoItemPO> metadataEntityInfoItemCol, Long directoryId, List<Long> infoItemIds, String entityTableName, String schemaName, Long pInfoItemId, String platCode, String tokenId, Integer isRes) throws Exception {
    List<MetadataEntityInfoItemPO> needSycInfoItemCol = new ArrayList<>();
    Boolean isExistField = Boolean.valueOf(createOperationTimeAndSyncStatus(entity, metadataEntityInfoItemCol, false, pInfoItemId, platCode, tokenId, isRes));
    if (!isExistField.booleanValue()) {
      MetadataEntityPO entityNew = ((IDataResource)ServiceUtil.getDataResourceManager(getDefaultCode(), IDataResource.class)).thisDirectoryExistMasterEntity(directoryId, platCode);
      metadataEntityInfoItemCol = ((IInfoItem)ServiceUtil.getInfoItemManager(getDefaultCode(), IInfoItem.class)).getEntityInfoItemsByEntityId(entityNew.getEntityId(), getDefaultCode(), isRes, tokenId);
      entity = entityNew;
    } 
    if (!StringUtils.isEmpty(entityTableName)) {
      if (StringUtils.isEmpty(schemaName)) {
        entity.setDataSchemaName(DEFAULT_SCHEMA);
      } else {
        entity.setDataSchemaName(schemaName);
      } 
      entity.setEntityTableName(entityTableName);
    } 
    List<MetadataEntityInfoItemPO> handInfoItems = metadataEntityInfoItemCol;
    if (pInfoItemId != null)
      handInfoItems = (List<MetadataEntityInfoItemPO>)metadataEntityInfoItemCol.stream().filter(i -> pInfoItemId.equals(i.getInfoItemParentId())).collect(Collectors.toList()); 
    for (MetadataEntityInfoItemPO item : handInfoItems) {
      if (((IInfoItemSync)ServiceUtil.getInfoItemSyncManager(getDefaultCode(), IInfoItemSync.class)).isSystemField(item.getInfoItemName()).booleanValue()) {
        needSycInfoItemCol.add(item);
        continue;
      } 
      if (infoItemIds.contains(item.getInfoItemId()) && item.getInfoItemType().intValue() != 1)
        needSycInfoItemCol.add(item); 
    } 
    if (needSycInfoItemCol.isEmpty())
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_SYNC_ENTITY_NO_INTES_FAILED.getCode()), StatusCodeEnum.DATA_RESOURCE_SYNC_ENTITY_NO_INTES_FAILED.getMessage()); 
    Boolean isExistSycItem = Boolean.valueOf(false);
    for (MetadataEntityInfoItemPO item : needSycInfoItemCol) {
      if (item.getDataItemId() != null) {
        MetadataEntityDataItemVO dataItem = (MetadataEntityDataItemVO)this.dataStandardClient.getMetadataEntityDataItemById(item.getDataItemId(), platCode, tokenId).getObj();
        if (dataItem != null && !StringUtils.isEmpty(item.getDataItemName()) && !StringUtils.isEmpty(dataItem.getDataType())) {
          isExistSycItem = Boolean.valueOf(true);
          break;
        } 
      } 
    } 
    if (!isExistSycItem.booleanValue())
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_SYNC_ENTITY_NO_DB_PROP_FAILED.getCode()), StatusCodeEnum.DATA_RESOURCE_SYNC_ENTITY_NO_DB_PROP_FAILED.getMessage()); 
    return ResultEntity.successMessage(needSycInfoItemCol);
  }
  
  private ResultEntity handleDBFieldNotExistsTable(MetadataEntityPO entity, List<MetadataEntityInfoItemPO> metadataEntityInfoItemCol, String entityTableName, String schemaName, String platCode, String tokenId, Boolean isSubInfoSync, DataStructProvider dataStructProvider, List<MetadataEntityInfoItemPO> primaryKeyInfoItemCol, Integer isRes) throws Exception {
    TableParamDTO tableParamDTO = new TableParamDTO();
    List<ColumnParamDTO> columns = getColumnParamDTOsFromInfoItems(metadataEntityInfoItemCol, dataStructProvider.currentDataSource.getType(), platCode, tokenId);
    tableParamDTO.setTableName(entityTableName);
    tableParamDTO.setColumns(columns);
    tableParamDTO.setSchemaName(dataStructProvider.currentDataSource.getDbSchema());
    Boolean isCreateTable = dataStructProvider.createTable(tableParamDTO);
    if (!isCreateTable.booleanValue())
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_CREATE_DB_TABLE_FAILED.getCode()), StatusCodeEnum.DATA_RESOURCE_CREATE_DB_TABLE_FAILED.getMessage()); 
    List<String> errorFields = new ArrayList<>();
    if (!StringUtils.isEmpty(entityTableName))
      if (isSubInfoSync.booleanValue()) {
        boolean isExistsOperationTime = false;
        Optional<MetadataEntityInfoItemPO> dbidOptional = metadataEntityInfoItemCol.stream().filter(c -> SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.DBID).equalsIgnoreCase(c.getInfoItemCode())).findFirst();
        MetadataEntityInfoItemPO metadataEntityInfoItem = (dbidOptional != null) ? dbidOptional.get() : null;
        if (metadataEntityInfoItem != null)
          isExistsOperationTime = true; 
        List<MetadataEntityInfoItemPO> childPkCols = new ArrayList<>();
        for (MetadataEntityInfoItemPO primaryKeyInfoItem : primaryKeyInfoItemCol) {
          MetadataEntityDataItemVO metadataEntityDataItem = (MetadataEntityDataItemVO)this.dataStandardClient.getMetadataEntityDataItemById(primaryKeyInfoItem.getDataItemId(), platCode, tokenId).getObj();
          boolean isAdd = true;
          if (SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.DBID).equalsIgnoreCase(primaryKeyInfoItem.getDataItemName()) && !isExistsOperationTime) {
            createPKColumnToChildTable(schemaName, entity, entityTableName, primaryKeyInfoItem, dataStructProvider, platCode, tokenId);
          } else if (primaryKeyInfoItem.getInfoItemParentId() == null) {
            isAdd = createPKColumnToChildTable(schemaName, entity, entityTableName, primaryKeyInfoItem, dataStructProvider, platCode, tokenId);
          } else if (primaryKeyInfoItem.getInfoItemParentId() != null) {
            childPkCols.add(primaryKeyInfoItem);
          } 
          if (!isAdd && !SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.DBID).equalsIgnoreCase(primaryKeyInfoItem.getInfoItemName()))
            errorFields.add(primaryKeyInfoItem.getInfoItemName()); 
        } 
        if (!childPkCols.isEmpty()) {
          Map<String, String> tableStruct = new HashMap<>();
          Map<String, Integer> tableIsNull = new HashMap<>();
          List<DBColumnDTO> tableStructDt = dataStructProvider.getEntityDbTableStruct(entityTableName, null);
          if (tableStructDt != null)
            for (DBColumnDTO dr : tableStructDt) {
              tableStruct.put(dr.getColumnName().toLowerCase(), dr.getDataType());
              tableIsNull.put(dr.getColumnName().toLowerCase(), dr.getIsNullAble());
            }  
          handlePrimaryKey(entity, entityTableName, schemaName, primaryKeyInfoItemCol, tableStruct, tableIsNull, dataStructProvider, platCode, tokenId, isSubInfoSync, Boolean.valueOf(false), isRes);
        } 
      } else {
        boolean isAdd = false;
        for (MetadataEntityInfoItemPO primaryKeyInfoItem : primaryKeyInfoItemCol) {
          isAdd = createColumn(primaryKeyInfoItem, dataStructProvider, entity, platCode, tokenId, true);
          if (!isAdd)
            errorFields.add(primaryKeyInfoItem.getInfoItemName()); 
        } 
      }  
    if (!errorFields.isEmpty())
      return returnErrorMessage(errorFields, platCode); 
    return ResultEntity.successMessage(Boolean.valueOf(true));
  }
  
  private boolean createPKColumnToChildTable(String schemaName, MetadataEntityPO entity, String entityTableName, MetadataEntityInfoItemPO primaryKeyInfoItem, DataStructProvider dataStructProvider, String platCode, String tokenId) throws Exception {
    if (StringUtils.isEmpty(schemaName)) {
      entity.setDataSchemaName(DEFAULT_SCHEMA);
    } else {
      entity.setDataSchemaName(schemaName);
    } 
    entity.setEntityTableName(entityTableName);
    if (primaryKeyInfoItem.getInfoItemParentId() == null)
      return syncParentFieldToChildTable(primaryKeyInfoItem, dataStructProvider, entity, platCode, tokenId, true); 
    return createColumn(primaryKeyInfoItem, dataStructProvider, entity, platCode, tokenId, true);
  }
  
  private ResultEntity handleDBFieldExistsTable(MetadataEntityPO entity, List<MetadataEntityInfoItemPO> metadataEntityInfoItemCol, String entityTableName, String schemaName, String platCode, String tokenId, Boolean isSubInfoSync, DataStructProvider dataStructProvider, List<MetadataEntityInfoItemPO> primaryKeyInfoItemCol, Integer isResource) throws Exception {
    Map<String, String> tableStruct = new HashMap<>();
    Map<String, Integer> tableIsNull = new HashMap<>();
    List<DBColumnDTO> tableStructDt = dataStructProvider.getEntityDbTableStruct(entityTableName, null);
    List<String> realPKColumns = new ArrayList<>();
    Boolean createPKDBID = Boolean.valueOf(false);
    if (tableStructDt != null)
      for (DBColumnDTO dr : tableStructDt) {
        tableStruct.put(dr.getColumnName().toLowerCase(), dr.getDataType());
        tableIsNull.put(dr.getColumnName().toLowerCase(), dr.getIsNullAble());
        if (dr.getIdentification().equals(Integer.valueOf(1)))
          realPKColumns.add(dr.getColumnName().toLowerCase()); 
      }  
    List<String> errorField = new ArrayList<>();
    for (MetadataEntityInfoItemPO infoItem : metadataEntityInfoItemCol) {
      try {
        if (!((IInfoItem)ServiceUtil.getInfoItemManager(getDefaultCode(), IInfoItem.class)).checkIsValidInfoItem(infoItem, isResource, tokenId))
          throw new Exception("无效的信息项，请检查是否配置数据类型和物理字段名称！"); 
        MetadataEntityDataItemVO dataItemVO = (MetadataEntityDataItemVO)this.dataStandardClient.getMetadataEntityDataItemById(infoItem.getDataItemId(), platCode, tokenId).getObj();
        if (tableStruct.containsKey(infoItem.getDataItemName().toLowerCase())) {
          if (infoItem.getDataItemName().equalsIgnoreCase(SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.DBID)))
            continue; 
          Integer isNull = tableIsNull.get(infoItem.getDataItemName().toLowerCase());
          if (realPKColumns.indexOf(infoItem.getDataItemName().toLowerCase()) == -1) {
            String dataType = ((IInfoItemSync)ServiceUtil.getInfoItemSyncManager(getDefaultCode(), IInfoItemSync.class)).getDBColumnType(dataStructProvider.currentDataSource.getType(), dataItemVO.getDataType(), platCode, tokenId);
            MetadataFieldVO metadataFieldVO = new MetadataFieldVO();
            BeanUtils.copyProperties(infoItem, metadataFieldVO);
            Boolean isAlter = Boolean.valueOf(dataStructProvider.alterColumn(schemaName, entityTableName, (dataItemVO.getDataLength() != null) ? dataItemVO.getDataLength().intValue() : 0, (dataItemVO.getDataScale() != null) ? dataItemVO.getDataScale().intValue() : 0, dataType, isNull.intValue(), metadataFieldVO));
            if (!isAlter.booleanValue())
              throw new Exception("修改列sql执行异常"); 
          } 
          if (isSubInfoSync.booleanValue()) {
            if (StringUtils.isEmpty(schemaName)) {
              entity.setDataSchemaName(DEFAULT_SCHEMA);
              continue;
            } 
            entity.setDataSchemaName(schemaName);
            continue;
          } 
          if (infoItem.getDataItemName().equalsIgnoreCase(SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.DBID)) && !realPKColumns.contains(SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.DBID).toLowerCase()))
            createPKDBID = Boolean.valueOf(true); 
          continue;
        } 
        if (infoItem.getDataItemName().equalsIgnoreCase(SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.DBID)))
          createPKDBID = Boolean.valueOf(true); 
        if (isSubInfoSync.booleanValue())
          if (StringUtils.isEmpty(schemaName)) {
            entity.setDataSchemaName(DEFAULT_SCHEMA);
          } else {
            entity.setDataSchemaName(schemaName);
          }  
        boolean isAdd = createColumn(infoItem, dataStructProvider, entity, platCode, tokenId, DCConst.TRUE.equals(infoItem.getIsRequired()));
        if (!isAdd)
          errorField.add(infoItem.getInfoItemName()); 
      } catch (Exception ex) {
        errorField.add(infoItem.getInfoItemName());
      } 
    } 
    ResultEntity handlePrimaryKeyRet = handlePrimaryKey(entity, entityTableName, schemaName, primaryKeyInfoItemCol, tableStruct, tableIsNull, dataStructProvider, platCode, tokenId, isSubInfoSync, createPKDBID, isResource);
    if (!errorField.isEmpty())
      return returnErrorMessage(errorField, platCode); 
    if (!handlePrimaryKeyRet.getCode().equals(Integer.valueOf(0)))
      return handlePrimaryKeyRet; 
    return ResultEntity.successMessage(Boolean.valueOf(true));
  }
  
  private ResultEntity returnErrorMessage(List<String> errorField, String platCode) {
    if ("Manage".equalsIgnoreCase(platCode))
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_SYNC_FIELD_FAILED.getCode()), "信息项【" + String.join(",", (Iterable)errorField) + "】同步失败"); 
    return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_SYNC_ITEM_FAILED.getCode()), "字段【" + String.join(",", (Iterable)errorField) + "】同步失败");
  }
  
  private void setDBFieldDescription(MetadataEntityPO entity, List<MetadataEntityInfoItemPO> metadataEntityInfoItemCol, String schemaName, String platCode, String tokenId, Boolean isSubInfoSync, DataSourceDTO dataSourceDTO) {
    for (MetadataEntityInfoItemPO infoItem : metadataEntityInfoItemCol) {
      if (infoItem.getInfoItemName().equalsIgnoreCase(SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.DBID)))
        continue; 
      try {
        MetadataEntityInfoItemPO item = infoItem;
        MetadataEntityDataItemVO dataItemVO = (MetadataEntityDataItemVO)this.dataStandardClient.getMetadataEntityDataItemById(item.getDataItemId(), platCode, tokenId).getObj();
        MetadataFieldVO itemInfoVO = new MetadataFieldVO();
        BeanUtils.copyProperties(item, itemInfoVO);
        String columnType = ((IInfoItemSync)ServiceUtil.getInfoItemSyncManager(getDefaultCode(), IInfoItemSync.class)).getDBColumnType(dataSourceDTO.getType(), dataItemVO.getDataType(), platCode, tokenId);
        if (isSubInfoSync.booleanValue())
          if (StringUtils.isEmpty(schemaName)) {
            entity.setDataSchemaName(DEFAULT_SCHEMA);
          } else {
            entity.setDataSchemaName(schemaName);
          }  
        ((DataStructProvider)DataStructProvider.getDataBaseProvider(dataSourceDTO).getObj()).setColumnDescription(dataItemVO, itemInfoVO, entity.getDataSchemaName(), entity.getEntityTableName(), columnType);
      } catch (Exception exception) {}
    } 
  }
  
  private boolean createOperationTimeAndSyncStatus(MetadataEntityPO entity, List<MetadataEntityInfoItemPO> infoItemCol, boolean operationTimeIsPrimaryKey, Long pInfoItemId, String platCode, String tokenId, Integer isRes) {
    boolean isExistField = false;
    boolean isExistOperationTime = false;
    boolean isExistSyncStatus = false;
    boolean isExistCreateTime = false;
    boolean isExistLastModifyTime = false;
    boolean isExistUserName = false;
    boolean isExistInterfaceId = false;
    MetadataEntityInfoItemPO infoItem = null;
    if (operationTimeIsPrimaryKey) {
      Optional<MetadataEntityInfoItemPO> optional1 = infoItemCol.stream().filter(i -> SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.DBID).equalsIgnoreCase(i.getInfoItemCode())).findFirst();
      infoItem = (optional1 != null && optional1.isPresent()) ? optional1.get() : null;
    } else {
      Optional<MetadataEntityInfoItemPO> optional1 = infoItemCol.stream().filter(i -> (SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.DBID).equalsIgnoreCase(i.getInfoItemCode()) && pInfoItemId.equals(i.getInfoItemParentId()))).findFirst();
      infoItem = (optional1 != null && optional1.isPresent()) ? optional1.get() : null;
    } 
    if (infoItem == null) {
      ((IInfoItemSync)ServiceUtil.getInfoItemSyncManager(getDefaultCode(), IInfoItemSync.class)).createInfoItemAndDataItem(entity, SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.DBID), 1, SysFieldHelper.getBaseSysFieldTypeName(BaseSysFieldEnum.DBID), 1, 0, platCode, pInfoItemId, tokenId, isRes);
    } else {
      isExistOperationTime = true;
      if (infoItem.getIsPrimaryKey().equals(DCConst.FALSE)) {
        infoItem.setIsPrimaryKey(DCConst.TRUE);
        ((IMetadataDao)ServiceUtil.getMetadataDao(platCode, IMetadataDao.class)).updateInfoItem(infoItem);
      } 
      MetadataEntityDataItemVO metadataEntityDataItemVO = null;
      if (infoItem.getDataItemId() != null)
        metadataEntityDataItemVO = (MetadataEntityDataItemVO)this.dataStandardClient.getMetadataEntityDataItemById(infoItem.getDataItemId(), platCode, tokenId).getObj(); 
      if (metadataEntityDataItemVO != null && !"Long".equalsIgnoreCase(metadataEntityDataItemVO.getDataType())) {
        metadataEntityDataItemVO.setDataType("Long");
        MetadataEntityDataItemDTO dataItemDTO = new MetadataEntityDataItemDTO();
        BeanUtils.copyProperties(metadataEntityDataItemVO, dataItemDTO);
        dataItemDTO.setIsDelete(DCConst.FALSE);
        this.dataStandardClient.updateMetadataEntityDataItemNotVerify(dataItemDTO, platCode, tokenId);
      } 
    } 
    Optional<MetadataEntityInfoItemPO> optional = infoItemCol.stream().filter(i -> SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.SYNC_STATUS).equalsIgnoreCase(i.getInfoItemCode())).findFirst();
    MetadataEntityInfoItemPO infoItemSyncStatus = (optional != null && optional.isPresent()) ? optional.get() : null;
    if (infoItemSyncStatus == null) {
      ((IInfoItemSync)ServiceUtil.getInfoItemSyncManager(getDefaultCode(), IInfoItemSync.class)).createInfoItemAndDataItem(entity, SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.SYNC_STATUS), 0, SysFieldHelper.getBaseSysFieldTypeName(BaseSysFieldEnum.SYNC_STATUS), 1, 0, platCode, pInfoItemId, tokenId, isRes);
    } else {
      isExistSyncStatus = true;
    } 
    optional = infoItemCol.stream().filter(c -> SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.START_TIME).equalsIgnoreCase(c.getInfoItemCode())).findFirst();
    MetadataEntityInfoItemPO infoItemCreateTime = (optional != null && optional.isPresent()) ? optional.get() : null;
    if (infoItemCreateTime == null) {
      ((IInfoItemSync)ServiceUtil.getInfoItemSyncManager(getDefaultCode(), IInfoItemSync.class)).createInfoItemAndDataItem(entity, SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.START_TIME), 0, SysFieldHelper.getBaseSysFieldTypeName(BaseSysFieldEnum.START_TIME), 1, 0, platCode, pInfoItemId, tokenId, isRes);
    } else {
      isExistCreateTime = true;
    } 
    optional = infoItemCol.stream().filter(c -> SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.START_TIME).equalsIgnoreCase(c.getInfoItemCode())).findFirst();
    MetadataEntityInfoItemPO infoItemLastModifyTime = (optional != null && optional.isPresent()) ? optional.get() : null;
    if (infoItemLastModifyTime == null) {
      ((IInfoItemSync)ServiceUtil.getInfoItemSyncManager(getDefaultCode(), IInfoItemSync.class)).createInfoItemAndDataItem(entity, SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.LAST_MODIFY_TIME), 0, SysFieldHelper.getBaseSysFieldTypeName(BaseSysFieldEnum.LAST_MODIFY_TIME), 1, 0, platCode, pInfoItemId, tokenId, isRes);
    } else {
      isExistLastModifyTime = true;
    } 
    optional = infoItemCol.stream().filter(c -> SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.USER_NAME).equalsIgnoreCase(c.getInfoItemCode())).findFirst();
    MetadataEntityInfoItemPO infoItemUserName = (optional != null && optional.isPresent()) ? optional.get() : null;
    if (infoItemUserName == null) {
      ((IInfoItemSync)ServiceUtil.getInfoItemSyncManager(getDefaultCode(), IInfoItemSync.class)).createInfoItemAndDataItem(entity, SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.USER_NAME), 0, SysFieldHelper.getBaseSysFieldTypeName(BaseSysFieldEnum.USER_NAME), 1, 0, platCode, pInfoItemId, tokenId, isRes);
    } else {
      isExistUserName = true;
    } 
    optional = infoItemCol.stream().filter(c -> SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.INTERFACE_ID).equalsIgnoreCase(c.getInfoItemCode())).findFirst();
    MetadataEntityInfoItemPO infoItemInterfaceId = (optional != null && optional.isPresent()) ? optional.get() : null;
    if (infoItemInterfaceId == null) {
      ((IInfoItemSync)ServiceUtil.getInfoItemSyncManager(getDefaultCode(), IInfoItemSync.class)).createInfoItemAndDataItem(entity, SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.INTERFACE_ID), 0, SysFieldHelper.getBaseSysFieldTypeName(BaseSysFieldEnum.INTERFACE_ID), 1, 0, platCode, pInfoItemId, tokenId, isRes);
    } else {
      isExistInterfaceId = true;
    } 
    if (isExistSyncStatus && isExistOperationTime && isExistCreateTime && isExistLastModifyTime && isExistUserName && isExistInterfaceId)
      isExistField = true; 
    return isExistField;
  }
  
  private ResultEntity handlePrimaryKey(MetadataEntityPO entity, String entityTableName, String schemaName, List<MetadataEntityInfoItemPO> primaryKeyInfoItemCol, Map<String, String> tableStruct, Map<String, Integer> tableIsNull, DataStructProvider dataStructProvider, String platCode, String tokenId, Boolean isSubInfoSync, Boolean createPKDBID, Integer isRes) throws Exception {
    if (!isSubInfoSync.booleanValue() && createPKDBID.booleanValue()) {
      List<Map<String, Object>> dataSet = dataStructProvider.getTableConstraintIndexName(entity.getEntityTableName(), "pk");
      if (dataSet != null && !dataSet.isEmpty() && 
        dataSet.get(0) != null && !((Map)dataSet.get(0)).isEmpty())
        for (String indexName : ((Map)dataSet.get(0)).keySet())
          dataStructProvider.delTablePK(entity.getEntityTableName(), ((Map)dataSet.get(0)).get(indexName).toString());  
      String columnType = ((IInfoItemSync)ServiceUtil.getInfoItemSyncManager(getDefaultCode(), IInfoItemSync.class)).getDBColumnType(dataStructProvider.currentDataSource.getType(), SysFieldHelper.getBaseSysFieldTypeName(BaseSysFieldEnum.DBID), platCode, tokenId);
      if (DCConst.PrimaryKeyMode.GUID.equals(this.primaryKeyMode))
        columnType = "char(36)"; 
      dataStructProvider.createColumn(entity.getEntityTableName(), SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.DBID), columnType, Integer.valueOf(1), Integer.valueOf(0));
      dataStructProvider.setTablePK(entity.getEntityTableName(), SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.DBID));
    } 
    if (!StringUtils.isEmpty(entityTableName)) {
      List<String> errorFields = new ArrayList<>();
      if (StringUtils.isEmpty(schemaName)) {
        entity.setDataSchemaName(DEFAULT_SCHEMA);
      } else {
        entity.setDataSchemaName(schemaName);
      } 
      for (MetadataEntityInfoItemPO primaryKeyInfoItem : primaryKeyInfoItemCol) {
        boolean syncRet = false;
        MetadataEntityDataItemVO metadataEntityDataItem = (MetadataEntityDataItemVO)this.dataStandardClient.getMetadataEntityDataItemById(primaryKeyInfoItem.getDataItemId(), platCode, tokenId).getObj();
        if (tableStruct.containsKey(primaryKeyInfoItem.getDataItemName().toLowerCase())) {
          if (primaryKeyInfoItem.getDataItemName().equalsIgnoreCase(SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.DBID)) || primaryKeyInfoItem.getInfoItemParentId() == null)
            continue; 
          tableStruct.get(primaryKeyInfoItem.getDataItemName().toLowerCase());
          Integer isNull = tableIsNull.get(primaryKeyInfoItem.getDataItemName().toLowerCase());
          String columnType = ((IInfoItemSync)ServiceUtil.getInfoItemSyncManager(getDefaultCode(), IInfoItemSync.class)).getDBColumnType(dataStructProvider.currentDataSource.getType(), metadataEntityDataItem.getDataType(), platCode, tokenId);
          syncRet = ((IInfoItem)ServiceUtil.getInfoItemManager(getDefaultCode(), IInfoItem.class)).alterColumn(primaryKeyInfoItem, metadataEntityDataItem, columnType, isNull.intValue(), dataStructProvider, entity, isRes, tokenId);
        } else {
          syncRet = syncParentFieldToChildTable(primaryKeyInfoItem, dataStructProvider, entity, platCode, tokenId, true);
        } 
        if (!syncRet && !SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.DBID).equalsIgnoreCase(primaryKeyInfoItem.getInfoItemName()))
          errorFields.add(primaryKeyInfoItem.getInfoItemName()); 
      } 
      if (!errorFields.isEmpty())
        return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_PRIMARY_KEY_SYNC_FAILED.getCode()), "同步出错，原因：主键字段【" + String.join(",", (Iterable)errorFields) + "】同步到数据库失败"); 
    } 
    return ResultEntity.successMessage(Boolean.valueOf(true));
  }
  
  private boolean syncParentFieldToChildTable(MetadataEntityInfoItemPO infoItem, DataStructProvider dataStructProvider, MetadataEntityPO entity, String platCode, String tokenId, boolean isPK) {
    try {
      MetadataEntityDataItemVO metadataEntityDataItem = (MetadataEntityDataItemVO)this.dataStandardClient.getMetadataEntityDataItemById(infoItem.getDataItemId(), platCode, tokenId).getObj();
      MetadataEntityPO mainEntity = ((IMetadataDao)ServiceUtil.getMetadataDao(platCode, IMetadataDao.class)).selectEntityByEntityId(infoItem.getEntityId());
      List<DBColumnDTO> allCols = dataStructProvider.getEntityDbTableStruct(mainEntity.getEntityTableName(), null);
      String columnType = "";
      if (allCols != null && !allCols.isEmpty())
        for (DBColumnDTO col : allCols) {
          if (col.getColumnName().equalsIgnoreCase(infoItem.getDataItemName())) {
            columnType = col.getFullColumnType();
            break;
          } 
        }  
      if (StringUtils.isEmpty(columnType))
        throw new Exception("主表的主键字段【" + infoItem.getInfoItemName() + "】未同步到主表的数据库，无法同步该字段到子表数据库中"); 
      return dataStructProvider.createColumn(entity.getEntityTableName(), infoItem.getDataItemName(), columnType, metadataEntityDataItem.getDataLength(), metadataEntityDataItem.getDataScale(), isPK).booleanValue();
    } catch (Exception e) {
      String msg = "信息项【" + infoItem.getInfoItemName() + "】同步到数据库失败：" + e.getMessage();
      logErrorInfo(ExceptionUtil.getStackTraceMsg(e), msg, platCode, tokenId);
      return false;
    } 
  }
  
  private boolean createColumn(MetadataEntityInfoItemPO infoItem, DataStructProvider dataStructProvider, MetadataEntityPO entity, String platCode, String tokenId, boolean isPK) {
    try {
      MetadataEntityDataItemVO metadataEntityDataItem = (MetadataEntityDataItemVO)this.dataStandardClient.getMetadataEntityDataItemById(infoItem.getDataItemId(), platCode, tokenId).getObj();
      String columnType = ((IInfoItemSync)ServiceUtil.getInfoItemSyncManager(getDefaultCode(), IInfoItemSync.class)).getDBColumnType(dataStructProvider.currentDataSource.getType(), metadataEntityDataItem.getDataType(), platCode, tokenId);
      return dataStructProvider.createColumn(entity.getEntityTableName(), infoItem.getDataItemName(), columnType, metadataEntityDataItem.getDataLength(), metadataEntityDataItem.getDataScale(), isPK).booleanValue();
    } catch (Exception e) {
      String msg = "信息项【" + infoItem.getInfoItemName() + "】同步到数据库失败：" + e.getMessage();
      logErrorInfo(ExceptionUtil.getStackTraceMsg(e), msg, platCode, tokenId);
      return false;
    } 
  }
  
  private List<ColumnParamDTO> getColumnParamDTOsFromInfoItems(List<MetadataEntityInfoItemPO> metadataEntityInfoItemCol, String dbType, String platCode, String tokenId) {
    List<ColumnParamDTO> columns = new ArrayList<>();
    for (int i = 0; i < metadataEntityInfoItemCol.size(); i++) {
      MetadataEntityInfoItemPO infoItem = metadataEntityInfoItemCol.get(i);
      if (infoItem.getDataItemId() != null) {
        MetadataEntityDataItemVO metadataEntityDataItem = (MetadataEntityDataItemVO)this.dataStandardClient.getMetadataEntityDataItemById(infoItem.getDataItemId(), platCode, tokenId).getObj();
        metadataEntityDataItem.setDataItemName(infoItem.getDataItemName());
        columns.add(((IInfoItem)ServiceUtil.getInfoItemManager(getDefaultCode(), IInfoItem.class)).createColumnParamDTO(metadataEntityDataItem, dbType, platCode, tokenId, infoItem.getIsPrimaryKey(), infoItem.getIsRequired()));
      } 
    } 
    return columns;
  }
  
  private boolean isSameTableName(Long dataSourceId, String tableName, Long entityId) {
    List<MetadataEntityPO> entityPOList = ((IMetadataDao)ServiceUtil.getMetadataDao(getDefaultCode(), IMetadataDao.class)).selectEntitysByDataSourceIdAndTableName(dataSourceId, tableName);
    if (entityPOList != null)
      for (int i = 0; i < entityPOList.size(); i++) {
        if (!((MetadataEntityPO)entityPOList.get(i)).getEntityId().equals(entityId))
          return true; 
      }  
    return false;
  }
  
  private void logErrorInfo(String stackInfo, String msg, String platCode, String tokenId) {
    CommonLogDTO logDTO = new CommonLogDTO();
    logDTO.setPlatCode(platCode);
    logDTO.setTokenId(tokenId);
    logDTO.setStackInformation(stackInfo);
    logDTO.setLoggerInformation(msg);
    logDTO.setLogLevel(FrameConst.Log.LOG_LEVEL_ERROR);
    logDTO.setLogType(0);
    logDTO.setModule_code(HeaderInterceptor.getModuleCode());
    RabbitMQUtil.publish("DC.Standard.log", logDTO);
  }
  
  private void logSystemInfo(String msg, String platCode, String tokenId) {
    CommonLogDTO logDTO = new CommonLogDTO();
    logDTO.setPlatCode(platCode);
    logDTO.setTokenId(tokenId);
    logDTO.setLoggerInformation(msg);
    logDTO.setLogLevel(FrameConst.Log.LOG_LEVEL_SYSTEM);
    logDTO.setLogType(0);
    logDTO.setModule_code(HeaderInterceptor.getModuleCode());
    RabbitMQUtil.publish("DC.Standard.log", logDTO);
  }
  
  private MetadataEntityInfoItemPO getItemInfoById(Long infoItemId, String tokenId, Integer isRes) {
    List<Long> ids = new ArrayList<>();
    ids.add(infoItemId);
    List<MetadataFieldWithElementVO> metadataFieldWithElementVOS = (List<MetadataFieldWithElementVO>)this.dataStandardClient.getFieldWithElementByIds(ids, isRes, getDefaultCode(), tokenId).getObj();
    MetadataEntityInfoItemPO metadataEntityInfoItem = this.metadataField.metadataFieldToInfoItem(metadataFieldWithElementVOS).get(0);
    return metadataEntityInfoItem;
  }
}
